@@ -3,7 +3,8 @@ package Getopt::Complete::Compgen;
 use strict;
 use warnings;
 
-our $VERSION = $Getopt::Complete::VERSION;
+use version;
+our $VERSION = qv('0.5');
 
 # Support the shell-builtin completions.
 # Some hackery seems to be required to replicate regular file completion.
@@ -31,19 +32,14 @@ for my $subname (qw/
         my @f =  grep { $_ !~/^\s+$/ } `bash -c "compgen -$option -- '$value'"`; 
         chomp @f;
         if ($option eq 'f' or $option eq 'd') {
-            # bash is fine with ~/ paths but perl is not, need to translate
-            my $home_dir = (getpwuid($<))[7];
-            for (my $i = 0; $i < @f; $i++) {
-                my $perl_path = $f[$i];
-                $perl_path =~ s/^~/$home_dir/;
-                if ( -d $perl_path ) {
-                    $f[$i] .= "/\t";
-                }
+            @f = map { -d $_ ? "$_/\t" : $_ } @f;
+            if (-d $value) {
+                push @f, [$value];
+                push @{$f[-1]},'-' if $Getopt::Complete::LONE_DASH_SUPPORT and $option eq 'f';
+            }
+            else {
+                push @f, ['-'] if $Getopt::Complete::LONE_DASH_SUPPORT and $option eq 'f';
             }
-
-            my @not_shown = ($value);
-            push @f, \@not_shown;
-            push @not_shown, '-' if $Getopt::Complete::LONE_DASH_SUPPORT and $option eq 'f';
         }
         return \@f;
     };
@@ -60,11 +56,11 @@ for my $subname (qw/
 
 =head1 NAME
 
-Getopt::Complete::Compgen - standard tab-completion callbacks
+Getopt::Complete::Compgen - built-in standard shell completions 
 
 =head1 VERSION
 
-This document describes Getopt::Complete::Compgen 0.25.
+This document describes Getopt::Complete::Compgen v0.5.
 
 =head1 SYNOPSIS
 
@@ -88,7 +84,7 @@ This document describes Getopt::Complete::Compgen 0.25.
 =head1  DESCRIPTION
 
 This module contains subroutines which can be used as callbacks with Getopt::Complete,
-and which implement all of the standard completions supported by the bash "compgen" builtin.
+and which implement all of the standard completions supported by "compgen"
 
 Running "compgen -o files abc" will produce the completion list as though the user typed "abc<TAB>",
 with the presumption the user is attempting to complete file names.
@@ -124,7 +120,7 @@ The manual page for bash details the bash built-in command "compgen", which this
 
 =head1 COPYRIGHT
 
-Copyright 2010 Scott Smith and Washington University School of Medicine
+Copyright 2009 Scott Smith and Washington University School of Medicine
 
 =head1 AUTHORS
 
