@@ -3,16 +3,14 @@ package Getopt::Complete::Options;
 use strict;
 use warnings;
 
-our $VERSION = $Getopt::Complete::VERSION;
+use version;
+our $VERSION = qv('0.5');
 
 use IPC::Open2;
-use Data::Dumper;
-use Getopt::Complete::LazyOptions;
 
 sub new {
     my $class = shift;
     my $self = bless {
-        sub_commands => [],
         option_specs => {},
         completion_handlers => {},
         parse_errors => undef,
@@ -21,11 +19,8 @@ sub new {
     # process the params into normalized completion handlers
     # if there are problems, the ->errors method will return a list.
     $self->_init(@_);
-    return $self;
-}
 
-sub sub_commands {
-    return @{ shift->{sub_commands} };
+    return $self;
 }
 
 sub option_names {
@@ -37,7 +32,6 @@ sub option_specs {
     my $self = shift;
     my @specs;
     for my $key (keys %{ $self->{option_specs} }) {
-        next if $key eq '<>';
         my $value = $self->{option_specs}{$key};
         push @specs, $key . $value;
     }
@@ -51,12 +45,6 @@ sub option_spec {
     return $self->{option_specs}{$name};
 }
 
-sub has_option {
-    my $self = shift;
-    my $name = shift;
-    return exists $self->{completion_handlers}{$name};
-}
-
 sub completion_handler {
     my $self = shift;
     my $name = shift;
@@ -74,7 +62,7 @@ sub _init {
     while (my $key = shift @_) {
         my $handler = shift @_;
         
-        my ($name,$spec) = ($key =~ /^([\w|-|\>][\w|-]*|\<\>|)(\W.*|)/);
+        my ($name,$spec) = ($key =~ /^([\w|-]+|\<\>|)(\W.*|)/);
         if (not defined $name) {
             push @parse_errors,  __PACKAGE__ . " is unable to parse '$key' from spec!";
             next;
@@ -101,32 +89,13 @@ sub _init {
                 $handler = $code;
             }
         }
-        if (substr($name,0,1) eq '>') {
-            # a "sub-command": make a sub-options tree, which may happen recursively
-            my $word = substr($name,1);
-            if (ref($handler) eq 'ARRAY') {
-                $handler = Getopt::Complete::Options->new(@$handler);
-            }
-            elsif (ref($handler) eq 'CODE' or ref($handler) eq 'SCALAR') {
-                # be lazy about actually resolving this               
-                $handler = Getopt::Complete::LazyOptions->new($handler);
-            }
-            else {
-                die "expected arrayref or code for $name value!";
-            }
-            $handler->{command} = ($self->{command} || '') . " " . $word; 
-            $completion_handlers->{$name} = $handler;
-            push @{ $self->{sub_commands} }, $word;
-            next;
-        }
-
         $completion_handlers->{$name} = $handler;
         if ($name eq '<>') {
             next;
         }
         if ($name eq '-') {
             if ($spec and $spec ne '!') {
-                push @parse_errors,  __PACKAGE__ . " $key errors: $name is implicitly stand-alone!";
+                push @parse_errors,  __PACKAGE__ . " $key errors: $name is implicitly boolean!";
             }
             $spec ||= '!';
         }
@@ -157,13 +126,8 @@ sub handle_shell_completion {
             exit;
         }
         my $args = Getopt::Complete::Args->new(options => $self, argv => $other);
-        my @matches;
-        my @printable_matches;
-        unless ($args->errors) {
-            @matches = $args->resolve_possible_completions($command,$current,$previous);
-            @printable_matches = $args->translate_completions_for_shell_display($current, @matches);
-        }
-        print join("\n",@printable_matches),"\n";
+        my @matches = $args->resolve_possible_completions($command,$current,$previous);
+        print join("\n",@matches),"\n";
         exit;
     }
     return 1;
@@ -180,14 +144,7 @@ sub _line_to_argv {
     my $result = join("",<$reader>);
     no strict; no warnings;
     my $array = eval $result;
-    my @array = @$array;
-
-    # We don't want to expand ~ for user experience and to be consistent with
-    # Bash's behavior for tab completion (as opposed to expansion of ARGV).
-    my $home_dir = (getpwuid($<))[7];
-    @array = map { $_ =~ s/^$home_dir/\~/; $_ } @array;
-
-    return @array;
+    return @$array;
 }
 
 sub parse_completion_request {
@@ -195,38 +152,30 @@ sub parse_completion_request {
     my ($comp_line, $comp_point) = @_;
 
     my $left = substr($comp_line,0,$comp_point);
-    my @left = _line_to_argv($left);
-
-    # find options for last sub-command if it has a completion handler
-    # skipping first command but old code didn't but it also never seemed to trigger before
-    my @sub_cmds = @left[1..$#left];
-    while (@sub_cmds and my $delegate = $self->completion_handler('>' . $sub_cmds[0])) {
-        shift @sub_cmds;
-        $self = $delegate;
-    }
-
     my $right = substr($comp_line,$comp_point);
+    
+    my @left = _line_to_argv($left);
     my @right = _line_to_argv($right);
     
     unless (@left) {
         # parse error
         return;
     }
- 
+
     my $command = shift @left;
     my $current;
-    if (substr($left, -1) ne ' ' || substr($left, -2) eq '\ ') {
+    if (@left and $left[-1] eq substr($left,-1*length($left[-1]))) {
         # we're at the end of the final word in the @left list, and are trying to complete it
         $current = pop @left;
     }
     else {
+        # we're starting to complete an empty word
         $current = '';
     }
-    $left =~ s/\\ / /g;
-    my $previous = ( (@left and $left[-1] =~ /^-{1,2}/ and not $left[-1] =~ /^-{1,2}[\w\-]+\=/) ? (pop @left) : ()) ;
-    # TODO: this might be a good spot to make sure we don't complete a new sub-command
+    my $previous = ( (@left and $left[-1] =~ /^--/) ? (pop @left) : ()) ;
     my @other_options = (@left,@right);
 
+
     # it's hard to spot the case in which the previous word is "boolean", and has no value specified
     if ($previous) {
         my ($name) = ($previous =~ /^-+(.*)/);
@@ -247,19 +196,9 @@ sub parse_completion_request {
         }
         
     }
-
-    my $quote;
-    if ($current =~ /^([\'\"])/) {
-        $quote = $1;
-        $current = substr($current,1);
-        if (substr($current,-1,1) eq $quote and not substr($current,-2,1) eq '\\') {
-           $current = substr($current,0,length($current)-1); 
-        };
-    }
-    return ($command,$current,$previous,\@other_options, $quote);
+    return ($command,$current,$previous,\@other_options);
 }
 
-1;
 
 =pod 
 
@@ -269,7 +208,7 @@ Getopt::Complete::Options - a command-line options specification
 
 =head1 VERSION
 
-This document describes Getopt::Complete 0.25
+This document describes Getopt::Complete v0.5.
 
 =head1 SYNOPSIS
 
@@ -317,7 +256,7 @@ L<Getopt::Complete>, L<Getopt::Complete::Args>, L<Getopt::Complete;:Compgen>
 
 =head1 COPYRIGHT
 
-Copyright 2010 Scott Smith and Washington University School of Medicine
+Copyright 2009 Scott Smith and Washington University School of Medicine
 
 =head1 AUTHORS
 
